local AddonName, GGL = ...
local Templates = GGL.Templates
local Session = Templates.Session

local Solo, SoloHelper = Templates:Register("Solo", "Session")

-- lua
local sub, match = string.sub, string.match

-- wow
local C_TimerAfter = C_Timer.After

-- ##################
-- local
-- ##################
local SessionSave = {}

-- set timers only one time
local TimerRegister = {}
local function Set_C_TimerAfter(after, method)
    if not TimerRegister[method] then
        TimerRegister[method] = true
        C_TimerAfter(after, method)
    end
end

-- ##################
-- static
-- ##################
local Static = {}
Templates.Solo = Static

function Static:GetSession(sessionDB, sessionID)
    sessionDB = sessionDB or GGL.sessionDb.Sessions
    if sessionID and SessionSave[sessionID] then
        return SessionSave[sessionID], false
    else
        return Templates:New("Solo", sessionDB, sessionID), true
    end
end

-- ##################
-- template
-- ##################
Solo.sessionFormat = "A%d"
Solo.sessionType = "A"

local function CB_SetDataRefresh(self)
    if self.topItemsFrame then
        self:RemoveFromTopItemsFrame()
        self:AddIntoTopItemsFrame(true)
    end
end


function Solo:Constructor(sessionDB, sessionID)
    if sessionID and SessionSave[sessionID] then
        return SessionSave[sessionID]
    end
    -- base
    self:CallBaseFunction("Session", "Constructor", sessionDB, sessionID)

    SessionSave[self.db.sessionID] = self
end

local function CB_CFrameOnNewItem(self, itemID, itemString, zoneID, sourceID, ammount, copperValue, mainSourceFlag, subSourceFlag, sourcePlayer)
    if self.topItemsFrame then
        self.topItemsFrame:AddItem(sourcePlayer, itemID, self:LoggedItemUseVariations(itemID, itemString) and itemString or nil, ammount, copperValue, zoneID, sourceID, mainSourceFlag, subSourceFlag)
    end
end

local function CB_CFrameOnAddMoney(self, copper, sourcePlayer)
    -- not needed now maybe later
    -- REGISTER CB WHEN NEEDED
end

-- #################
-- TopItems
-- #################
function Solo:RemoveFromTopItemsFrame()
    if not self.topItemsFrame then return end
    local frame = self.topItemsFrame

    -- Unregister all player events
    frame:UnregisterAllPlayerEvents()

    -- remove from minimap button
    GGL.GUI.TopItems:RemoveFromMiniMapButton()

    -- callback
    self:UnregisterCallbackEvent("SessionTemplate_AddItem", CB_CFrameOnNewItem)
    --self:UnregisterCallbackEvent("SessionTemplate_AddMoney", CB_CFrameOnAddMoney)

    self.topItemsFrame = nil
end

function Solo:AddIntoTopItemsFrame(add)
    if not self.db or self.db.isClosed then return end

    local frame = GGL.GUI.TopItems:Show()

    -- clear all
    frame:Clear()

    -- set session
    frame:SetSession(self)

    -- update
    frame:UpdateMergeList()

    -- add into small frame
    GGL.GUI.TopItems:AddIntoMiniMapButton()

    -- callback
    self:RegisterCallbackEvent("SessionTemplate_AddItem", CB_CFrameOnNewItem)
    --self:RegisterCallbackEvent("SessionTemplate_AddMoney", CB_CFrameOnAddMoney)

    self.topItemsFrame = frame
end

-- #################
-- Solo
-- #################
function Solo:Start()
    -- always show main and Solo frame
    self:AddIntoMainFrame(true)
    self:AddIntoTopItemsFrame(true)

    self:CallBaseFunction("Session", "Start")
    if not self.db.isRunning then return end

    -- SetDataRefresh
    self:RegisterCallbackEvent("SessionTemplate_SetDataRefresh", CB_SetDataRefresh)

    -- update
    if self.topItemsFrame then
        self.topItemsFrame:UpdateMergeList()
    end

    return self.db.startTime, self.db.endTime, self.db.durationTime
end

function Solo:SetData(mainSet, setRefresh)
    self:CallBaseFunction("Session", "SetData", mainSet, setRefresh)
end

local function UnregisterAll(self)
    SessionSave[self.db.sessionID] = nil
end

function Solo:Unregister()
    self:CallBaseFunction("Session", "Unregister")

    UnregisterAll(self)
end

function Solo:Stop()
    self:CallBaseFunction("Session", "Stop")

    UnregisterAll(self)

    SessionSave[self.db.sessionID] = self
end

--- Close the session
function Solo:Close()
    self:CallBaseFunction("Session", "Close")

    UnregisterAll(self)
end