local AddonName, GGL = ...
local Templates = GGL.Templates

local ZoneFilter, ZoneFilterHelper = Templates:Register("ZoneFilter")

-- lua
local bit_band = bit.band

-- wow
local GetBestMapForUnit = C_Map.GetBestMapForUnit
local EJ_GetInstanceForMap = EJ_GetInstanceForMap or GGL.EJ_GetInstanceForMap
local GetInstanceInfo = GetInstanceInfo

-- ##################
-- local
-- ##################
local CONTINENT_MAP_LIST = GGL.CONTINENT_MAP_LIST
local MAP_ID_DATA = GGL.MAP_ID_DATA
local MAP_BASE_ID = GGL.MAP_BASE_ID
local INSTANCE_ID_DATA = GGL.INSTANCE_ID_DATA
local ZONE_FILTER_IGNORE = GGL.ZONE_FILTER_IGNORE

local function LoadSetDB(self, id, key)
    if not key then return end
    if key == true then return true end
    if type(key) == "table" then return key end
    local db = GGL.db.Config.Sets[id]
    if db and db[key] then
        local pass, tableDB = GGL:Deserialize(db[key].setting)
        return tableDB or {}
    end
end

local function LoadSetDB_Setting(self, setting)
    if not setting then return end
    if setting == true then return true end
    local pass, tableDB = GGL:Deserialize(setting)
    return tableDB or {}
end

-- ##################
-- static
-- ##################
local Static = {}
Templates.ZoneFilter = Static

function Static.IgnoreZoneFilter(mainSourceFlag, subSourceFlag)
    if not mainSourceFlag and not subSourceFlag then return end

    if ZONE_FILTER_IGNORE[mainSourceFlag] then
        if ZONE_FILTER_IGNORE[mainSourceFlag] == true or ZONE_FILTER_IGNORE[mainSourceFlag][subSourceFlag] then
            return true
        end
    end
end

-- ##################
-- template
-- ##################
function ZoneFilter:Constructor(zoneSetKey, instanceSetKey, isSettingString)
    if isSettingString then
        self:LoadSetSettings(zoneSetKey, instanceSetKey)
    else
        self:LoadSet(zoneSetKey, instanceSetKey)
    end

end

function ZoneFilter:LoadSet(zoneSetKey, instanceSetKey)
    self.zoneSetKey = zoneSetKey
    self.instanceSetKey = instanceSetKey
    self.db = {
        zone = LoadSetDB(self, GGL.SET_ID_LIST.Zone, zoneSetKey),
        instance = LoadSetDB(self, GGL.SET_ID_LIST.Instance, instanceSetKey)
    }
end

function ZoneFilter:LoadSetSettings(zoneSetting, instanceSetting)
    self.zoneSetKey = nil
    self.instanceSetKey = nil
    self.db = {
        zone = LoadSetDB_Setting(self, zoneSetting),
        instance = LoadSetDB_Setting(self, instanceSetting)
    }
end

function ZoneFilter:ReloadSet()
    if not self.zoneSetKey and not self.instanceSetKey then return end
    self:LoadSet(self.zoneSetKey, self.instanceSetKey)
end

function ZoneFilter:IgnoreZoneFilter(mainSourceFlag, subSourceFlag)
    return Static.IgnoreZoneFilter(mainSourceFlag, subSourceFlag)
end

local MissingLog = {zone={}, instance={}}
function ZoneFilter:IsValidZone(zoneID, difficultyID, mainSourceFlag, subSourceFlag)
    if GGL.IS_BCC or GGL.IS_CLASSIC then return true end   -- TODO: add  zone filter support
    if Static.IgnoreZoneFilter(mainSourceFlag, subSourceFlag) then return true end
    zoneID = zoneID or GetBestMapForUnit("player")
    if not zoneID then return end -- happens sometimes on enter a instance
    local instanceID = GGL.FIX_MAPID_TO_INSTANCID[zoneID] or EJ_GetInstanceForMap(zoneID)

    if self.db.instance and instanceID and instanceID > 0 then
        if self.db.instance == true then
            return true
        end
        local expansionID, _
        -- try get current diff
        if not difficultyID then
            _, _, difficultyID = GetInstanceInfo()
        end
        difficultyID = instanceID..":"..(difficultyID or 1)

        if not INSTANCE_ID_DATA[instanceID] then
            if not MissingLog.instance[instanceID] then
                MissingLog.instance[instanceID] = true
                GGL:AddIntoErrorLog("ZoneFilter:IsValidZone", format("Missing instanceID '%d' forund.", instanceID or 0))
            end
        elseif INSTANCE_ID_DATA[difficultyID] then
            expansionID = INSTANCE_ID_DATA[difficultyID][1]
        else
            expansionID = INSTANCE_ID_DATA[instanceID][1]
        end

        -- instance
        local db = self.db.instance

        -- check full expansionID fist
        if db.e and db.e[expansionID] == true then
            return false
        elseif db.e and db.e[expansionID] then
            if INSTANCE_ID_DATA[difficultyID] then
                if not db[difficultyID] then
                    return false
                end
            elseif not db[instanceID] then
                return false
            end
        end
    elseif self.db.zone then
        if self.db.zone == true then
            return true
        end
        -- normal zone
        zoneID = MAP_BASE_ID[zoneID] or zoneID
        if not MAP_ID_DATA[zoneID] then
            if not not MissingLog.zone[zoneID] then
                MissingLog.zone[zoneID] = zoneID and C_Map.GetMapInfo(zoneID) or true
                GGL:AddIntoErrorLog("ZoneFilter:IsValidZone", format("Missing zoneID '%d' forund", instanceID or 0))
            end
            return
        end

        -- its a zone
        local db = self.db.zone
        local continentID = MAP_ID_DATA[zoneID][2]

        -- continent forbidden
        if db[continentID] == true then
            return false
        end

        -- zone forbidden
        if db[continentID] and not db[continentID][zoneID] then
            return false
        end
    end
    return true
end